@@ -3,11 +3,15 @@ from .listexternal import print_contents, get_xml_files, get_sample_files, get_c
 from .indata import get_default_data, get_data
 from .read_mappings import get_mapping,get_all_mappings,get_biocatch_mappings,get_socure_mappings,get_biocatch_mappings
 from .read_mappings import read_to_sas_mappings,get_payfoneverify_mappings,get_mapping_choices,add_mapping
+from .read_mappings import get_giact_mappings, get_payfonetrust_mappings, get_bokugpir_mappings
+from .read_mappings import get_bokumaa_mappings, get_datavisor_mappings, get_iovation_mappings, add_mapping_in_str
 from .steps import get_steps
 from .ccexception import CustomCallException
 from .targets import set_default_targets, get_targets, get_target, add_or_update_target, set_enrich_targets
+from .targets import add_get_target,add_post_target,add_target_with_map_str
 from .runsteps import decide_with_json
 from .tosas import convert_to_sas
 from .main import enrich
 from .trial import add_one
-from .generate_id_code import gen_structure
+from .generate_python_id_code import gen_python_structure
+from .generate_ds2_id_code import gen_ds2_structure
