@@ -2,6 +2,8 @@ import os
 
 from enrichwrap import enrich, add_mapping
 
+dotted_sep = '   # ............................................................................................................'
+dashed_sep = '   # ------------------------------------------------------------------------------------------------------------'
 
 def get_return_line(extra, sas_data):
     val = ['   return ']
@@ -32,29 +34,38 @@ def get_starter_body(tool, http_url=None, http_port=None, vendor_required_data_i
     if vendor_required_data_in is not None:
         vendor_required_data = vendor_required_data_in
 
-    content = ["",
-               "# -------------------------------------------------------------------------------------------",
+    content = ["", dashed_sep,
                "# TODO: Either import the decision 'variables-biocatch' or complete the subsequent TODO items",
-               "# -------------------------------------------------------------------------------------------",
-               "",
-               "# ...........................................................................................",
-               "# TODO: Go to the variables tab, and ensure that vendor_list is a Character",
-               "# ...........................................................................................",
-               "   vendor_list = enrichwrap.get_mapping_vendor_list()", ""]
+               dashed_sep, "", dotted_sep, "   #    Specify the data to use for the enrichment", dotted_sep]
 
     if vendor_required_data is not None:
         content.append("   vendor_required_data = '" + vendor_required_data_in + "'")
     else:
-        content.append("   vendor_required_data = None")
+        content.append("   data = []")
+        content.append("   data.append('{\"_1_app_account_balance\": 23696')")
+        content.append("   data.append(' \"_1_app_account_holder_fullnm\": \"Kennith\"')")
+        content.append("   data.append(' \"_1_app_account_num\": \"19204353938236134\"}')")
+        content.append("   vendor_required_data = ','.join(data)")
     content.append("")
 
+    content.append(dotted_sep)
+    content.append("   #    Specify the target machine on which the third party enrichment services are running")
+    content.append(dotted_sep)
+    if http_url is not None:
+        content.append("   targets = enrichwrap.set_default_targets('" + http_url + "', '" + http_port + "')")
+    else:
+        content.append("   targets = enrichwrap.set_default_targets('10.44.16.24', '8200')")
+
     if targets_in is not None:
         content.append("   targets = enrichwrap.add_or_update_target('" + tool + "',")
         content.append("             " + str(targets_in[tool]) + ")")
-    else:
-        content.append("   targets = None")
     content.append("")
 
+    content.append(dotted_sep)
+    content.append("   #     Use the default (provided) mappings")
+    content.append("   #        - from vendor_required_data to the third party enrichment")
+    content.append("   #        - from the third party enrichment to the variables that we can export from this node")
+    content.append(dotted_sep)
     if mappings_in is not None and mappings_out is not None:
         content.append("   to_map = " + str(mappings_in))
         content.append("   from_map = " + str(mappings_out))
@@ -63,14 +74,14 @@ def get_starter_body(tool, http_url=None, http_port=None, vendor_required_data_i
         content.append("   mappings = None")
     content.append("")
 
-    content.append("   val = enrichwrap.enrich('" + tool + "', vendor_required_data, None, targets, mappings)")
+    content.append("   val = enrichwrap.enrich('" + tool + "', vendor_required_data, targets, mappings)")
     content.append("   sas_data = val['sas_data']")
     content.append("   vendor_response = val['vendor_response']")
     content.append("")
-    content.append("   # ............................................................................................................")
-    content.append("   # TODO: Go to the Variables tab, and ensure that vendor_list, dict and success_of_failure_rc are set to Character")
-    content.append("   # ............................................................................................................")
-    content.append("   dict = str(sas_data)")
+    content.append(dotted_sep)
+    content.append("   # TODO: Go to the Variables tab, and ensure that sas_data_json and success_or_failure_rc are set to Character")
+    content.append(dotted_sep)
+    content.append("   sas_data_json = str(sas_data)")
     content.append("")
     content.append("   # Comes back as success or failure")
     content.append("   success_or_failure_rc = vendor_response['" + tool + "']['result']")
@@ -85,41 +96,39 @@ def get_variables(sas_data):
     for key in sas_data.keys():
         val = sas_data[key]
         if isinstance(val, bool):
-            bools.append("   " + key + " = (sas_data['" + key + "'] is True)")
+            bools.append("   " + key + " = (sas_data.get('" + key + "', False) is True)")
 
     # Find the integers
     for key in sas_data.keys():
         val = sas_data[key]
         if isinstance(val, bool) == False and isinstance(val, int):
-            ints.append("   " + key + " = sas_data['" + key + "']")
+            ints.append("   " + key + " = sas_data.get('" + key + "', 0)")
 
     # Find the strings
     for key in sas_data.keys():
         val = sas_data[key]
         if isinstance(val, str):
-            strings.append("   " + key + " = sas_data['" + key + "']")
+            strings.append("   " + key + " = sas_data.get('" + key + "', '')")
 
     if len(bools) > 0:
-        content.append('   # .....................................................................................................')
+        content.append(dotted_sep)
         content.append('   # TODO: Go to the Variables tab, and ensure the following are mapping to Boolean')
-        content.append('   # .....................................................................................................')
+        content.append(dotted_sep)
         content.append('   # Booleans')
         content.append('\n'.join(bools))
         content.append('')
 
     if len(ints) > 0:
-        content.append('   # .....................................................................................................')
+        content.append(dotted_sep)
         content.append('   # TODO: Go to the Variables tab, and ensure the following are mapping to Integer')
-        content.append('   # .....................................................................................................')
-        content.append('   # Integers')
+        content.append(dotted_sep)
         content.append('\n'.join(ints))
         content.append('')
 
     if len(strings) > 0:
-        content.append('   # .....................................................................................................')
+        content.append(dotted_sep)
         content.append('   # TODO: Go to the Variables tab, and ensure the following are mapping to Strings')
-        content.append('   # .....................................................................................................')
-        content.append('   # Strings')
+        content.append(dotted_sep)
         content.append('\n'.join(strings))
         content.append('')
 
@@ -150,7 +159,7 @@ def gen_python_structure(tool, mappings, vendor_required_data=None, targets=None
         outgoing_data = enrich(tool, vendor_required_data, targets, created_mappings)
 
     sas_data = outgoing_data['sas_data']
-    extra = 'vendor_list,success_or_failure_rc,dict'
+    extra = 'success_or_failure_rc,sas_data_json'
 
     id_content = ["import enrichwrap",
                   '',
